/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "ftp_main.h"

#define CAPA_MAX	10

static struct {
	char* capa[CAPA_MAX];
	int ncapas;
} capas;

void capa_init() {
	memset(&capas, 0, sizeof(capas));
	capas.ncapas = 0;
}

char* capa_get_as_char() {
	int i;
	char* s;
	if (capas.ncapas == 0) {
		s = strdup("No special capabilities.");
		enough_mem(s);
	} else {
		s = strdup(capas.capa[0]);
		enough_mem(s);
		for (i = 1; i < capas.ncapas; i++) {
			size_t size = strlen(s)  /* existing */
				+ 1              /* space */
				+ strlen(capas.capa[i])
				+ 1;             /* termination */
			s = (char*) realloc(s, size);
			enough_mem(s);
			strcat(s, " ");
			strcat(s, capas.capa[i]);
		}
	}
	return s;
}

int capa_add(const char* capa) {
	if (capas.ncapas == CAPA_MAX) {
		return -1;
	}
	capas.capa[capas.ncapas] = strdup(capa);
	enough_mem(capas.capa[capas.ncapas]);
	capas.ncapas++;
	return 0;
}

/* the return value is either 0 or 1 or a negative value in case of an error */
int capa_has_server_capa(char* capa, struct proxyinfo* pi,
					struct messagebuffer* msgbuf) {

	struct ftp_message msg;
	int ss = pi->si->connection.serversocket;
	int cs = pi->si->connection.clientsocket;
	char* haystack, *needle, *msg_no_codes;
	int found;

	say(ss, "CAPA\r\n");
	msg = ftp_readall(ss);
	if (msg.error) {
		ftp_message_say_error(msg, msgbuf, cs, __FILE__, __LINE__);
		return TRANSFERMODE_FATAL;;
	}

	if (msg.code >= 500) {
		ftp_message_free(&msg);
		jlog(9, "Server doesn't seem to know the CAPA command");
		return TRANSFERMODE_NOTSUPPORTED;
	}

	msg_no_codes = strip_respcodes(msg.msg);
	term_before_newline(msg_no_codes);
	ftp_message_free(&msg);
	haystack = char_enclose(" ", msg_no_codes, " ");
	free(msg_no_codes);

	needle = char_enclose(" ", capa, " ");
	found = strstr(haystack, needle) ? 1 : 0;
	free(haystack);
	free(needle);
	if (found) {
		jlog(9, "Found CAPA %s", capa);
		return TRANSFERMODE_OK;
	} else {
		jlog(9, "Could not find CAPA %s", capa);
		return TRANSFERMODE_NOTSUPPORTED;
	}
}
