/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#ifndef __FTP_TRANSFERMODE_H__
#define __FTP_TRANSFERMODE_H__

/* #define TRANSFER_BUFFERSIZE (MIN_VAL(65535, SSIZE_MAX)) */
#define TRANSFER_BUFFERSIZE 65535

#define TRANSFERMODE_OK			0
#define TRANSFERMODE_NOTSUPPORTED	(-1)
#define TRANSFERMODE_FAIL		(-2)
#define TRANSFERMODE_FATAL		(-3) /* terminates the session */

#define	TRANSFERMODE_RECV_SERVER	0
#define	TRANSFERMODE_RECV_CLIENT	1
#define	TRANSFERMODE_SEND_SERVER	2
#define	TRANSFERMODE_SEND_CLIENT	3

struct messagebuf;
struct ftp_message;

typedef int(*transfermode_func_t)
			(int, struct transfer_t*, struct proxyinfo*);
typedef int(*transfermode_init_func_t)
			(const char* cmd, const char** args,
			 	struct proxyinfo*, struct messagebuffer*,
				struct ftp_message*);
typedef int(*transfermode_sorry_client_func_t)
			(const char* cmd, const char** args,
			 	struct proxyinfo*, struct messagebuffer*,
				struct ftp_message*);
typedef int(*transfermode_deinit_func_t)
			(struct proxyinfo*, struct messagebuffer*,
				struct ftp_message*);

struct transfermode_t {
	int id;
	char* name;
	int type;
	int priority; /* the higher the number, the higher the priority */
	transfermode_func_t transfermode_func;
	transfermode_init_func_t transfermode_init_func;
	transfermode_deinit_func_t transfermode_deinit_func;
	transfermode_sorry_client_func_t transfermode_sorry_client_func;
	char* description;
};

int transfermode_init(void);
int transfermode_deinit(void);

const struct transfermode_t* transfermode_get(int type, int least_id);
const struct transfermode_t* transfermode_get_by_name(int type, const char* name);

int transfermode_register(int priority,
			int type,
			const char* name,
			transfermode_func_t transfermode_func,
			transfermode_init_func_t,
			transfermode_deinit_func_t,
			transfermode_sorry_client_func_t,
			const char* description);


/* Transfermode STD */

int transfermode_std_in(int, struct transfer_t*, struct proxyinfo*);
int transfermode_std_out(int, struct transfer_t*, struct proxyinfo*);

int transfermode_std_in_from_server_init(const char* cmd,
					 const char** args,
					 struct proxyinfo* pi,
					 struct messagebuffer* msgbuf,
					 struct ftp_message* msg);

int transfermode_std_in_from_client_init(const char* cmd,
					 const char** args,
					 struct proxyinfo* pi,
					 struct messagebuffer* msgbuf,
					 struct ftp_message* msg);

int transfermode_std_out_to_server_init(const char* cmd,
					const char** args,
					struct proxyinfo* pi,
					struct messagebuffer* msgbuf,
					struct ftp_message* msg);

int transfermode_std_out_to_client_init(const char* cmd,
					const char** args,
					struct proxyinfo* pi,
					struct messagebuffer* msgbuf,
					struct ftp_message* msg);

int transfermode_std_deinit_server(struct proxyinfo* pi,
				   struct messagebuffer* msgbuf,
				   struct ftp_message* msg);

int transfermode_std_deinit_client(struct proxyinfo* pi,
				   struct messagebuffer* msgbuf,
				   struct ftp_message* msg);

int transfermode_std_sorry_client(const char* cmd,
				  const char** args,
				  struct proxyinfo* pi,
				  struct messagebuffer* msgbuf,
				  struct ftp_message* msg);


#endif

