/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "jproxygw.h"
#include "unistd.h"
#include "signal.h"

static
int auth_call_external(const char* request,
				const char* prog, const char* args[]) {
	char* env[] = { (char*) 0 };
	int fd1[2] = { -1, -1 };
	int fd2[2] = { -1, -1 };
	char buf[2] = { 'E', 'R' };
	pid_t pid;

	if (pipe(fd1) < 0 || pipe(fd2) < 0) {
		jlog(2, "Could not create a pipe to run the authenticator: %s ",
							strerror(errno));
		return AUTH_FAIL;
	}
	pid = fork();
	if (pid < 0) {
		jlog(2, "Could not fork to run authenticator: %s",
							strerror(errno));
		close(fd1[0]); close(fd1[1]);
		close(fd2[0]); close(fd2[1]);
		return AUTH_FAIL;
	}
	if (pid == 0) {
		/* we are the child */
		/* we (child) write to fd1 and read from fd2 */
		close(fd1[0]); close(fd2[1]);
		dup2(fd1[1], STDOUT_FILENO);
		dup2(fd2[0], STDIN_FILENO);
		execve(prog, (char** const) args, env);
		jlog(2, "Could not run authenticator \"%s\": %s",
						prog, strerror(errno));
		exit(1);
	}
	if (pid > 0) {
		struct sigaction old_act, new_act;
		/* we (parent) write to fd2 and read from fd1 */
		close(fd1[1]); close(fd2[0]);

		new_act.sa_handler = SIG_IGN;
		sigemptyset(&new_act.sa_mask);
		new_act.sa_flags = 0;
		sigaction(SIGPIPE, &new_act, &old_act);
		/* XXX make configurable: auth.timeout */
		alarm(5);
		if (write(fd2[1], request, strlen(request)) < 0)
			goto deinit;
		if (write(fd2[1], "\n", 1) < 0)
			goto deinit;
		if (read(fd1[0], buf, 2) < 0)
			goto deinit;

		/* success */
deinit:
		alarm(0);
		sigaction(SIGPIPE, &old_act, NULL);
		close(fd1[0]); close(fd2[1]);
		if (buf[0] == 'O' && buf[1] == 'K') {
			return AUTH_OK;
		}
	}

	return AUTH_FAIL;
}

static
int auth_validate_extended(struct option_t* context,
				const char* prog, const char* args[]) {
	/* XXX */
	return AUTH_FAIL;
}

static
int auth_validate_simple(struct option_t* context,
				const char* prog, const char* args[]) {

	const char* username = option_get_value(context, "username");
	const char* password = option_get_value(context, "password");

	char* request = char_enclose(username, " ", password);

	int result = auth_call_external(request, prog, args);

	if (result == AUTH_OK) {
		jlog(8, "Authentication of user %s ok", username);
	} else {
		jlog(8, "Authentication of user %s failed", username);
	}
	overwrite_string(request);
	free(request);

	return result;
}

int auth_validate_external(struct option_t* context, int proglevel,
				const char* prog, const char** args_p) {
	/* const char* args[]; */
	/* = { args_p[0] }; */
	if (proglevel == AUTH_LEVEL_SIMPLE) {
		return auth_validate_simple(context, prog, args_p);
	} else {
		return auth_validate_extended(context, prog, args_p);
	}
}

